/*
 * JORAM: Java(TM) Open Reliable Asynchronous Messaging
 * Copyright (C) 2021 ScalAgent Distributed Technologies
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s):  ScalAgent Distributed Technologies
 * Contributor(s): 
 */
package joram.reconf;

import java.io.File;
import java.util.StringJoiner;

import org.objectweb.joram.client.jms.admin.AdminModule;

import fr.dyade.aaa.agent.AgentServer;
import framework.TestCase;

/**
 * Tests the update of the stored configuration.
 */
public class UpdateConfTest0 extends TestCase {
  public static void main(String[] args) {
    new UpdateConfTest0().run();
  }
  
  public void run() {
    try {
      // Remove configuration file from framework.
      new File("./a3servers.xml").delete();
      
      String config = generateA3Config("root", "root", 2560);
      // Try to update stored configuration, fails because the server has never been initialized. 
      int res = AgentServer.updateConfFromString(config, "./s0");
      assertEquals(res, -1);
      
      AgentServer.setDefaultConfig(config);
      AgentServer.init(new String[] {"0", "./s0"});
      AgentServer.start();

      // Check root credential's and listening port
      AdminModule.connect("localhost", 2560, "root", "root", 30);
      AdminModule.disconnect();
      
      AgentServer.stop();
      Exception exc = null;
      try {
        res = AgentServer.updateConfFromString(config, "./s0");
      } catch (Exception e) {
        exc =e;
      }
      assertNotNull(exc);
      AgentServer.reset();
      
      config = generateA3Config("root", "admin", 2561);
      res = AgentServer.updateConfFromString(config, "./s0");
      assertEquals(res, 0);
      AgentServer.init(new String[] {"0", "./s0"});
      exc = null;
      try {
        res = AgentServer.updateConfFromString(config, "./s0");
      } catch (Exception e) {
        exc =e;
      }
      assertNotNull(exc);
      AgentServer.start();
      exc = null;
      try {
        res = AgentServer.updateConfFromString(config, "./s0");
      } catch (Exception e) {
        exc =e;
      }
      assertNotNull(exc);

      // Check root credential's and listening port
      AdminModule.connect("localhost", 2561, "root", "admin", 30);
      AdminModule.disconnect();

      AgentServer.stop();
    } catch (Throwable exc) {
      exc.printStackTrace();
      error(exc);
    } finally {
      endTest();
    }
  }

  public static String generateA3Config(String admin, String password, int port) {
    StringJoiner config = new StringJoiner(System.lineSeparator());

    config.add("<config>");
    config.add("<property name=\"Transaction\" value=\"fr.dyade.aaa.ext.NGTransaction\"/>");
    config.add("<server id=\"0\" name=\"S0\" hostname=\"localhost\">");
    config.add("<service class=\"org.objectweb.joram.mom.proxies.ConnectionManager\" args=\"" + admin + " " + password + "\"/>");
    config.add("<service class=\"org.objectweb.joram.mom.proxies.tcp.TcpProxyService\" args=\"" + port + "\"/>");

    config.add("</server>");
    config.add("</config>");
    return config.toString();
  }

}

